<?php

namespace Ems\portalGestor\Controller;

use Ems\portalGestor\Service\DemonstrativoService;
use Zeedhi\Framework\Controller\CrudNext;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;
use Zeedhi\Framework\DTO\Response\{ Error, Message };

class DemonstrativoController {
    /** @var DemonstrativoService */
    protected $demonstrativoService;

    public function __construct(
        DemonstrativoService $demonstrativoService
    ) {
        $this->demonstrativoService = $demonstrativoService;
    }

    // x
    public function getDiasDemonstrativo(
        DTO\Request\Filter $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getParameters();
    
            $demonstrativo = $this->demonstrativoService->getDiasDemonstrativo($row);
    
            $response->addDataSet(new DataSet('data', $demonstrativo));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function getPeriodosDemonstrativo(
        DTO\Request\Filter $request,
        DTO\Response $response
    ) {
        $row = $request->getParameters();

        $periodos = $this->demonstrativoService->getPeriodosDemonstrativo($row);

        $response->addDataSet(new DataSet('data', $periodos));
    }

    public function getVinculosDoGestor(
        DTO\Request\Filter $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg = (int) $row['NRORG'];

            $nrEstrutura = (int) $row['NRESTRUTURAM'];

            // Deve ser passado o perído desejado OU o tipo do vínculo
            $nrPeriodo   = isset($row['NRPERIODOAPURACAO']) ?
                (int) $row['NRPERIODOAPURACAO'] :
                -1; // no webservice, o "-1" será substituído pelo período atual

            $nrTipoVinculo   = isset($row['NRTIPOVINCULOM']) ?
                (int) $row['NRTIPOVINCULOM'] :
                1;

            $vinculos = $this->demonstrativoService->getVinculosDoGestor($nrOrg, $nrEstrutura, $nrPeriodo, $nrTipoVinculo);

            foreach($vinculos as &$vinculo) {
                $vinculo['NRVINCULOM'] = (int) $vinculo['NRVINCULOM'];
            }

            $response->addDataSet(new DataSet('data', $vinculos));
        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getMotivosOcorrencia(
        DTO\Request\Filter $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getParameters();
            $motivos = $this->demonstrativoService->getMotivosOcorrencia($row);
            $response->addDataSet(new DataSet('data', $motivos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function retrocederApuracaoPeriodo(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();
            $retrocesso = $this->demonstrativoService->retrocederApuracaoPeriodo($row);

            $dataSet = [
                'retrocesso' => $retrocesso
            ];

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function apurarPeriodo(
        DTO\Request\Row $request,
        DTO\Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrOrg     = $row['NRORG'];
            $nrVinculo = $row['NRVINCULOM'];
            $nrPeriodo = $row['NRPERIODOAPURACAO'];
            $nrOrg = $row['NRORG'];

            $dataSet = $this->demonstrativoService->apurarPeriodo($nrOrg, $nrVinculo, $nrPeriodo);

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getParesMarcacoesDia(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg     = (int) $row['NRORG'];
            $nrVinculo = (int) $row['NRVINCULOM'];
            $dia       = $row['DIA'];

            $dataSet = $this->demonstrativoService->
                getParesMarcacoesDia($nrOrg, $nrVinculo, $dia);

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function removeParMarcacao(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrOrg             = (int) $row['NRORG'];
            $nrVinculo         = (int) $row['NRVINCULOM'];
            $nrMarcacaoEntrada = (int) $row['NRMARCACAOENTRADA'];
            $nrMarcacaoSaida   = (int) $row['NRMARCACAOSAIDA'];

            $marcacoesRemovidas = $this->demonstrativoService->
                removeParMarcacao($nrOrg, $nrVinculo, $nrMarcacaoEntrada, $nrMarcacaoSaida);

            $dataSet = [
                'marcacoesRemovidas' => $marcacoesRemovidas
            ];

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function editParMarcacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrOrg             = (int) $row['NRORG'];
            $nrVinculo         = (int) $row['NRVINCULOM'];
            $nrMarcacaoEntrada = (int) $row['NRMARCACAOENTRADA'];
            $nrMarcacaoSaida   = (int) $row['NRMARCACAOSAIDA'];
            $dia               = (string) $row['DIA'];
            $hrEntrada         = (string) $row['HRENTRADA'];
            $hrSaida           = (string) $row['HRSAIDA'];
            $idDiaInicio       = (string) $row['DIA_INICIO'];
            $idDiaFim          = (string) $row['DIA_FIM'];

            $nrMarcacoes = $this->demonstrativoService->editParMarcacao(
                $nrOrg, $nrVinculo, $nrMarcacaoEntrada,
                $nrMarcacaoSaida, $dia, $hrEntrada, $hrSaida,
                $idDiaInicio, $idDiaFim
            );

            $dataSet = [
                'nrMarcacoes' => $nrMarcacoes
            ];

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function saveParMarcacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrOrg             = (int) $row['NRORG'];
            $nrVinculo         = (int) $row['NRVINCULOM'];
            $dia               = (string) $row['DIA'];
            $hrEntrada         = (string) $row['HRENTRADA'];
            $hrSaida           = (string) $row['HRSAIDA'];
            $idDiaInicio       = (string) $row['DIA_INICIO'];
            $idDiaFim          = (string) $row['DIA_FIM'];

            $nrMarcacoes = $this->demonstrativoService->saveParMarcacao(
                $nrOrg, $nrVinculo, $dia, $hrEntrada,
                $hrSaida, $idDiaInicio, $idDiaFim
            );

            $dataSet = [
                'nrMarcacoes' => $nrMarcacoes
            ];

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function saveParesMarcacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrOrg             = (int)      $row['NRORG'];
            $nrVinculo         = (int)      $row['NRVINCULOM'];
            $dia               = (array)    $row['DIA'];
            $hrEntrada         = (string)   $row['HRENTRADA'];
            $hrSaida           = (string)   $row['HRSAIDA'];
            $idDiaInicio       = (string)   $row['DIA_INICIO'];
            $idDiaFim          = (string)   $row['DIA_FIM'];

            $nrMarcacoes = $this->demonstrativoService->saveParesMarcacao(
                $nrOrg, $nrVinculo, $dia, $hrEntrada,
                $hrSaida, $idDiaInicio, $idDiaFim
            );

            $dataSet = [
                'nrMarcacoes' => $nrMarcacoes
            ];

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function solicitaAbonoFaltas(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrOrg      = (int)     $row['NRORG'];
            $nrVinculo  = (int)     $row['NRVINCULOM'];
            $date       = (array)   $row['DATE'];
            $observacao = (string)  $row['DSOBSERVACAO'];
            $file       = (string)  $row['FILE'];
            $cdTabecdi      = (string)  isset($row['CDTABECDI'])   ? $row['CDTABECDI']     : null;
            $cdDiagnost     = (string)  isset($row['CDDIAGNOST'])  ? $row['CDDIAGNOST']    : null;
            $nrPessoa       = (string)  isset($row['NRPESSOA'])    ? $row['NRPESSOA']      : null;

            $success = $this->demonstrativoService->solicitaAbonoFaltas(
                $nrOrg, $nrVinculo, $date,
                $observacao, $file, $cdTabecdi,
                $cdDiagnost, $nrPessoa
            );

            $dataset = [
                'success' => $success,
            ];

            $response->addDataSet(new DataSet('data', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function solicitarCompensacaoHoras(
        Request\Row     $request,
        Response        $response
    ) {

        try {
            $row = $request->getRow();

            $nrOrg          = (int)     $row['NRORG'];
            $nrVinculo      = (int)     $row['NRVINCULOM'];
            $dia            = (string)  $row['DATE'];

            $status = $this->demonstrativoService->solicitarCompensacaoHoras(
                $nrOrg, $nrVinculo, $dia
            );

            if ($status['success']) {
                $dataSet = [
                    'success' => true
                ];

                $response->addDataSet(new DataSet('data', $dataSet));
            } else {
                $message = $status['message'];
                $response->addMessage(new Message($message, Message::TYPE_ERROR));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function solicitaCompensacaoFaltas(
        DTO\Request\Row     $request,
        DTO\Response        $response
    ) {

        try {
            $row = $request->getRow();

            $nrOrg          = (int)     $row['NRORG'];
            $nrVinculo      = (int)     $row['NRVINCULOM'];
            $dia            = (array)   $row['DATE'];

            $status = $this->demonstrativoService->solicitaCompensacaoFaltas(
                $nrOrg,
                $nrVinculo,
                $dia
            );

            if ($status['success']) {
                $dataSet = [
                    'success' => true
                ];

                $response->addDataSet(new DataSet('data', $dataSet));
            } else {
                $message = $status['message'];
                $response->addMessage(new Message($message, Message::TYPE_ERROR));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function solicitarHorasExtras(
        Request\Row     $request,
        Response        $response
    ) {

        try {
            $row = $request->getRow();

            $nrOrg          = (int)     $row['NRORG'];
            $nrVinculo      = (int)     $row['NRVINCULOM'];
            $observacao     = (string)  $row['DSOBSERVACAO'];
            $dia            = (string)  $row['DATE'];

            $status = $this->demonstrativoService->solicitarHorasExtras(
                $nrOrg, $nrVinculo, $observacao, $dia
            );

            if ($status['success']) {
                $dataSet = [
                    'success' => true
                ];

                $response->addDataSet(new DataSet('data', $dataSet));
            } else {
                $message = $status['message'];
                $response->addMessage(new Message($message, Message::TYPE_ERROR));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setTrocasHorario(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrOrg      = (int)     $row['NRORG'];
            $nrVinculo  = (int)     $row['NRVINCULOM'];
            $date       = (array)   $row['DATE'];
            $observacao = (string)  $row['DSOBSERVACAO'];
            $nrHordiam  = (string)  $row['NRHORDIAM'];
            $file       = (string)  isset($row['FILE']) ? $row['FILE'] : null;

            $success = $this->demonstrativoService->setTrocasHorario(
                $nrOrg, $nrVinculo, $date,
                $observacao, $nrHordiam, $file
            );

            $dataset = [
                'success' => $success,
            ];

            $response->addDataSet(new DataSet('data', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
}
